from __future__ import annotations

import os
from importlib.metadata import PackageNotFoundError, version

import httpx

from .resources.marketplace import MarketplaceResource
from .resources.runs import RunsResource
from .resources.storage import StorageResource

DEFAULT_BASE_URL = "https://api.stewai.com/v1"
ENV_API_KEY = "STEWAI_API_KEY"


def _package_version() -> str:
    try:
        return version("stewai")
    except PackageNotFoundError:  # pragma: no cover
        return "0.0.0"


class Client:
    """StewAI API client."""

    def __init__(
        self,
        api_key: str | None = None,
        *,
        base_url: str = DEFAULT_BASE_URL,
        timeout: float = 60.0,
        transport: httpx.BaseTransport | None = None,
    ) -> None:
        resolved_key = api_key or os.environ.get(ENV_API_KEY)
        if not resolved_key:
            raise ValueError(
                f"API key required: pass api_key= or set {ENV_API_KEY} environment variable"
            )

        base_url = base_url.rstrip("/") + "/"
        self._client = httpx.Client(
            base_url=base_url,
            timeout=timeout,
            transport=transport,
            headers={
                "Authorization": f"Bearer {resolved_key}",
                "User-Agent": f"stewai-python/{_package_version()}",
            },
        )
        self.runs = RunsResource(self._client)
        self.marketplace = MarketplaceResource(self._client, self.runs)
        self.storage = StorageResource(self._client)

    def close(self) -> None:
        self._client.close()

    def __enter__(self) -> "Client":
        return self

    def __exit__(self, exc_type, exc, tb) -> None:
        self.close()


Stew = Client



from importlib.metadata import PackageNotFoundError, version

from .client import Client, Stew
from .errors import (
    ApiError,
    AuthenticationError,
    RateLimitError,
    RunBlockedError,
    RunCancelledError,
    RunError,
    RunFailedError,
    StewAIError,
    StewAITimeoutError,
    StewError,
)
from .types import (
    IngestSource,
    MarketplaceInputContract,
    MarketplaceInputContractField,
    MarketplaceListResponse,
    MarketplaceRecipeDetail,
    MarketplaceRecipeSearchItem,
    MarketplaceRecipeSummary,
    MarketplaceSearchResponse,
    UploadResult,
)

try:
    __version__ = version("stewai")
except PackageNotFoundError:  # pragma: no cover
    __version__ = "0.0.0"

__all__ = [
    "Client",
    "Stew",
    "__version__",
    "StewAIError",
    "StewError",
    "ApiError",
    "AuthenticationError",
    "RateLimitError",
    "RunError",
    "RunFailedError",
    "RunCancelledError",
    "RunBlockedError",
    "StewAITimeoutError",
    "IngestSource",
    "UploadResult",
    "MarketplaceInputContractField",
    "MarketplaceInputContract",
    "MarketplaceRecipeSummary",
    "MarketplaceRecipeDetail",
    "MarketplaceRecipeSearchItem",
    "MarketplaceListResponse",
    "MarketplaceSearchResponse",
]



from __future__ import annotations

from dataclasses import dataclass
from typing import Any, Literal, Optional, TypedDict


@dataclass(frozen=True)
class IngestSource:
    uri: str
    kind: Literal["url", "file"] = "url"
    label: Optional[str] = None

    def to_dict(self) -> dict:
        data = {"uri": self.uri, "kind": self.kind}
        if self.label:
            data["label"] = self.label
        return data


@dataclass(frozen=True)
class UploadResult:
    obj_ulid: str
    uri: str
    filename: str
    size_bytes: int

    def to_ingest_source(self, label: Optional[str] = None) -> IngestSource:
        return IngestSource(uri=self.uri, kind="file", label=label)


class MarketplaceInputContractField(TypedDict, total=False):
    id: str
    label: str
    type: str
    required: bool
    max_length: int | None
    enum: list[str] | None


class MarketplaceInputContract(TypedDict):
    fields: list[MarketplaceInputContractField]


class MarketplaceRecipeSummary(TypedDict, total=False):
    recipe_ulid: str
    title: str
    description: str | None
    visibility: str
    listed: bool
    is_composable: bool
    revision_sha: str
    published_at: str
    total_runs: int
    success_rate: float | None
    avg_credits_per_run: float | None
    avg_duration_ms: float | None
    avg_rating: float | None
    rating_count: int
    safety_status: str
    marketplace: dict[str, Any]


class MarketplaceRecipeDetail(MarketplaceRecipeSummary, total=False):
    contract_preview: dict[str, Any]
    input_contract: MarketplaceInputContract


class MarketplaceRecipeSearchItem(MarketplaceRecipeSummary, total=False):
    score: float


class MarketplaceListResponse(TypedDict):
    count: int
    next: str | None
    previous: str | None
    results: list[MarketplaceRecipeSummary]


class MarketplaceSearchResponse(TypedDict):
    count: int
    next: str | None
    previous: str | None
    results: list[MarketplaceRecipeSearchItem]



from __future__ import annotations

from dataclasses import dataclass
from typing import Any, Optional


class StewAIError(Exception):
    """Base exception for all StewAI SDK errors."""


StewError = StewAIError


@dataclass
class ApiError(StewAIError):
    """HTTP-level API error."""

    status_code: int
    body: Any = None
    message: str = "API request failed"


class AuthenticationError(ApiError):
    """401/403 authentication failure."""


class RateLimitError(ApiError):
    """429 rate limit exceeded."""

    retry_after: Optional[str] = None


@dataclass
class RunError(StewAIError):
    """Base class for run-level errors."""

    run_id: str
    run: dict


class RunFailedError(RunError):
    """Run completed with status='failed'."""


class RunCancelledError(RunError):
    """Run was cancelled (status='cancelled')."""


class RunBlockedError(RunError):
    """Run blocked due to insufficient credits (status='blocked')."""


@dataclass
class StewAITimeoutError(StewAIError):
    """SDK-level timeout waiting for run completion."""

    run_id: str
    timeout: float
    last_status: str
    message: str = "Timed out waiting for run"
